@@ -7,7 +7,7 @@ Attean::API - Utility package for loading all Attean role packages.
 
 =head1 VERSION
 
-This document describes Attean::API version 0.029
+This document describes Attean::API version 0.031
 
 =head1 SYNOPSIS
 
@@ -25,15 +25,24 @@ in the Attean::API namespace.
 
 =cut
 
-package Attean::API::ResultOrTerm 0.029 {
+package Attean::API::ResultOrTerm 0.031 {
 	use Moo::Role;
 }
 
-package Attean::API::BlankOrIRI 0.029 {
+package Attean::API::BlankOrIRI 0.031 {
+	use Moo::Role;
+	with 'Attean::API::Term', 'Attean::API::BlankOrIRIOrTriple';
+}
+
+package Attean::API::BlankOrIRIOrTriple 0.031 {
 	use Moo::Role;
 }
 
-package Attean::API::TermOrVariable 0.029 {
+package Attean::API::TermOrTriple 0.031 {
+	use Moo::Role;
+}
+
+package Attean::API::TermOrVariable 0.031 {
 	use Scalar::Util qw(blessed);
 	use Sub::Install;
 	use Sub::Util qw(set_subname);
@@ -88,12 +97,63 @@ package Attean::API::TermOrVariable 0.029 {
 	}
 }
 
-package Attean::Mapper 0.029 {
+package Attean::API::TermOrVariableOrTriplePattern 0.031 {
+	use Scalar::Util qw(blessed);
+	use Sub::Install;
+	use Sub::Util qw(set_subname);
+
+	use Moo::Role;
+
+	with 'Attean::API::SPARQLSerializable';
+
+	sub is_bound {
+	  my $self = shift;
+	  return (! $self->does('Attean::API::Variable'));
+	}
+	
+	sub apply_binding {
+		my $self	= shift;
+		my $class	= ref($self);
+		my $bind	= shift;
+		if ($self->does('Attean::API::Variable')) {
+			my $name	= $self->value;
+			my $replace	= $bind->value($name);
+			if (defined($replace) and blessed($replace)) {
+				return $replace;
+			} else {
+				return $self;
+			}
+		} else {
+			return $self;
+		}
+	}
+
+	BEGIN {
+		my %types	= (
+			variable	=> 'Variable',
+			blank		=> 'Blank',
+			literal		=> 'Literal',
+			resource	=> 'IRI',
+			iri			=> 'IRI',
+			pattern		=> 'TriplePattern'
+		);
+		while (my ($name, $role) = each(%types)) {
+			my $method	= "is_$name";
+			my $code	= sub { return shift->does("Attean::API::$role") };
+			Sub::Install::install_sub({
+				code	=> set_subname($method, $code),
+				as		=> $method
+			});
+		}
+	}
+}
+
+package Attean::Mapper 0.031 {
 	use Moo::Role;
 	requires 'map'; # my $that = $object->map($this)
 }
 
-package Attean::API::Variable 0.029 {
+package Attean::API::Variable 0.031 {
 	use AtteanX::SPARQL::Constants;
 	use AtteanX::SPARQL::Token;
 
@@ -120,7 +180,7 @@ Returns a string representation of the variable.'
 	
 }
 
-package Attean::API::CanonicalizingBindingSet 0.029 {
+package Attean::API::CanonicalizingBindingSet 0.031 {
 	use Attean::RDF;
 
 	use Moo::Role;
@@ -217,7 +277,7 @@ package Attean::API::CanonicalizingBindingSet 0.029 {
 	}
 }
 
-package Attean::API 0.029 {
+package Attean::API 0.031 {
 	use Attean::API::Term;
 	use Attean::API::Store;
 	use Attean::API::Model;
@@ -255,7 +315,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2020 Gregory Todd Williams.
+Copyright (c) 2014--2022 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
