@@ -7,17 +7,30 @@ AtteanX::Parser::NQuads - N-Quads Parser
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::NQuads version 0.030
+This document describes AtteanX::Parser::NQuads version 0.031
 
 =head1 SYNOPSIS
 
  use Attean;
  my $parser = Attean->get_parser('NQuads')->new();
+
+ # Parse data from a file-handle and handle quads in the 'handler' callback
  $parser->parse_cb_from_io( $fh );
+ 
+ # Parse the given byte-string, and return an iterator of quads
+ my $iter = $parser->parse_iter_from_bytes('<http://example.org/subject> <tag:example.org:predicate> "object" <http://example.org/graph> .');
+ while (my $quad = $iter->next) {
+   print $quad->as_string;
+ }
 
 =head1 DESCRIPTION
 
-...
+This module implements a parser for the N-Quads format.
+
+=head1 ROLES
+
+This class consumes L<Attean::API::Parser>, L<Attean::API::PullParser>
+and <Attean::API::MixedStatementParser>.
 
 =head1 METHODS
 
@@ -25,7 +38,7 @@ This document describes AtteanX::Parser::NQuads version 0.030
 
 =cut
 
-package AtteanX::Parser::NQuads 0.030 {
+package AtteanX::Parser::NQuads 0.031 {
 	use utf8;
 	
 	use Attean;
@@ -93,7 +106,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2020 Gregory Todd Williams. This
+Copyright (c) 2014--2022 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
