@@ -7,7 +7,7 @@ AtteanX::Parser::Turtle::Constants - Constant definitions for use in parsing Tur
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::Turtle::Constants version 0.029
+This document describes AtteanX::Parser::Turtle::Constants version 0.031
 
 =head1 SYNOPSIS
 
@@ -19,7 +19,7 @@ This document describes AtteanX::Parser::Turtle::Constants version 0.029
 
 =cut
 
-package AtteanX::Parser::Turtle::Constants 0.029 {
+package AtteanX::Parser::Turtle::Constants 0.031 {
 	use v5.14;
 	use warnings;
 
@@ -43,6 +43,7 @@ package AtteanX::Parser::Turtle::Constants 0.029 {
 			LBRACE
 			LBRACKET
 			LPAREN
+			GRAPH
 			PREFIX
 			PREFIXNAME
 			RBRACE
@@ -55,6 +56,10 @@ package AtteanX::Parser::Turtle::Constants 0.029 {
 			STRING3S
 			TURTLEBASE
 			TURTLEPREFIX
+			GTGT
+			LTLT
+			LANNOT
+			RANNOT
 			WS
 			decrypt_constant
 		)
@@ -101,7 +106,7 @@ Toby Inkster C<< <tobyink@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2020 Toby Inkster. This
+Copyright (c) 2014--2022 Toby Inkster. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
