@@ -7,7 +7,7 @@ AtteanX::Parser::Turtle::Token - Token objects used for parsing of Turtle
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::Turtle::Token version 0.030
+This document describes AtteanX::Parser::Turtle::Token version 0.031
 
 =head1 SYNOPSIS
 
@@ -64,9 +64,10 @@ use Types::Standard qw(ArrayRef Str);
 use List::MoreUtils qw(zip);
 use Sub::Util qw(set_subname);
 use AtteanX::Parser::Turtle::Constants;
+use Sub::Install;
 use namespace::clean;
 
-our $VERSION	= 0.030;
+our $VERSION	= 0.031;
 
 has type => ( is => 'ro', );
 has start_line => ( is => 'ro', );
@@ -134,6 +135,24 @@ sub fast_constructor {
 	}
 }
 
+=item C<< as_string >>
+
+Returns a string description of the token including the token type and any
+associated values.
+
+=cut
+
+sub as_string {
+	my $self	= shift;
+	my $type	= decrypt_constant($self->type);
+	my @args	= @{ $self->args };
+	if (scalar(@args)) {
+		return "$type(" . join(', ', @args) . ")";
+	} else {
+		return $type;
+	}
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -155,7 +174,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2020 Gregory Todd Williams.
+Copyright (c) 2014--2022 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
