@@ -7,7 +7,7 @@ AtteanX::SPARQL::Token - Token objects used for parsing and serializing SPARQL
 
 =head1 VERSION
 
-This document describes AtteanX::SPARQL::Token version 0.029
+This document describes AtteanX::SPARQL::Token version 0.031
 
 =head1 SYNOPSIS
 
@@ -55,7 +55,7 @@ An array of values associated with the token (e.g. the integer value of an INT t
 
 =cut
 
-package AtteanX::SPARQL::Token 0.029;
+package AtteanX::SPARQL::Token 0.031;
 
 use Moo;
 use Types::Standard qw(ArrayRef Str);
@@ -113,7 +113,6 @@ sub fast_constructor {
 		semicolon	=> [SEMICOLON, ';'],
 		hathat		=> [HATHAT, '^^'],
 
-
 		lbrace		=> [LBRACE, '{'],
 		rbrace		=> [RBRACE, '}'],
 		op_andand	=> [ANDAND, '&&'],
@@ -133,6 +132,12 @@ sub fast_constructor {
 		question	=> [QUESTION, '?'],
 		slash		=> [SLASH, '/'],
 		star		=> [STAR, '*'],
+
+		ltlt		=> [LTLT, '<<'],
+		gtgt		=> [GTGT, '>>'],
+		lannot		=> [LANNOT, '{|'],
+		rannot		=> [RANNOT, '|}'],
+		
 	);
 	for my $name (keys %tokens) {
 		my ($type, $value)	= @{ $tokens{ $name } };
@@ -212,7 +217,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014--2020 Gregory Todd Williams.
+Copyright (c) 2014--2022 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
